@@ -7,19 +7,19 @@ use space::{MetricPoint, Neighbor};
 struct Euclidean(&'static [f64]);
 
 impl MetricPoint for Euclidean {
+    type Metric = u64;
     fn distance(&self, rhs: &Self) -> u64 {
-        space::f64_metric(
-            self.0
-                .iter()
-                .zip(rhs.0.iter())
-                .map(|(&a, &b)| (a - b).powi(2))
-                .sum::<f64>()
-                .sqrt(),
-        )
+        self.0
+            .iter()
+            .zip(rhs.0.iter())
+            .map(|(&a, &b)| (a - b).powi(2))
+            .sum::<f64>()
+            .sqrt()
+            .to_bits()
     }
 }
 
-fn test_hnsw() -> (Hnsw<Euclidean, Pcg64, 12, 24>, Searcher) {
+fn test_hnsw() -> (Hnsw<Euclidean, Pcg64, 12, 24>, Searcher<u64>) {
     let mut searcher = Searcher::default();
     let mut hnsw = Hnsw::new();
 
@@ -50,7 +50,10 @@ fn insertion() {
 fn nearest_neighbor() {
     let (hnsw, mut searcher) = test_hnsw();
     let searcher = &mut searcher;
-    let mut neighbors = [Neighbor::invalid(); 8];
+    let mut neighbors = [Neighbor {
+        index: !0,
+        distance: !0,
+    }; 8];
 
     hnsw.nearest(
         &Euclidean(&[0.0, 0.0, 0.0, 1.0]),
@@ -73,31 +76,31 @@ fn nearest_neighbor() {
             },
             Neighbor {
                 index: 4,
-                distance: 1072693248
+                distance: 4607182418800017408
             },
             Neighbor {
                 index: 7,
-                distance: 1072693248
+                distance: 4607182418800017408
             },
             Neighbor {
                 index: 1,
-                distance: 1073127582
+                distance: 4609047870845172685
             },
             Neighbor {
                 index: 2,
-                distance: 1073127582
+                distance: 4609047870845172685
             },
             Neighbor {
                 index: 3,
-                distance: 1073127582
+                distance: 4609047870845172685
             },
             Neighbor {
                 index: 5,
-                distance: 1073460858
+                distance: 4610479282544200874
             },
             Neighbor {
                 index: 6,
-                distance: 1073460858
+                distance: 4610479282544200874
             }
         ]
     );
